      module eigen_house
      implicit NONE

         real(8), pointer       ::  u0_z(:), v0_z(:)
         real(8), pointer       ::  u1_z(:), v1_z(:)
         integer                ::  offset1, offset2
         integer                ::  offset3, offset4

      end module eigen_house

