#imrcolor,lincolorは2通りの指定方法(linecolor,linecolor2)のうち片方を使う
erange=" -erange -9,26"
#erange=" -erange 0,4"
einc="-einc 1.0"
#imrfont="-imrfont Helvetica,11"
#ticsfont="-ticsfont Helvetica,10"
#imrcolor="-imrcolor #545422"
imrcolor2="-imrcolor blue"
#linecolor="-linecolor #545422"
linecolor2="-linecolor red"
imrtype="-imrtype mulliken"
#noncross="-nonecross"
#kgrouptype="-kgrouptype Schoenflies"
withfermi="-with_fermi nfefermi.data"
numimr="-numimr 2"
perl band_symm_new02.pl reduce_down.data ${erange} ${einc} ${imrfont} ${ticsfont} ${imrcolor} ${imrcolor2} ${linecolor} ${linecolor2} ${imrtype} ${noncross} ${kgrouptype} ${withfermi} ${numimr}
#perl ${HOME}/band_symm/perl/band_symm.pl reduce.data ${erange} ${einc} ${imrfont} ${ticsfont} ${imrcolor} ${imrcolor2} ${linecolor} ${linecolor2} ${imrtype} ${noncross} ${kgrouptype} ${withfermi} ${numimr}
