!
! =========== Contributions ===================================
!
! Through the courtesy of contributors, the following functions are added.
!
! Company:  ASMS Co.,Ltd.
! Functions:  [Identifier: 13.1R]
!            Dielectric functions can be calculated when condition = INITIAL 
!
! Functions:  [Identifier: 13.1XI]
!            Dielectric functions can be calculated in the EXCITATION module
!
! =============================================================
subroutine Epsilon_Postscf


end subroutine Epsilon_Postscf

