!#========================================================================
!#                                                                       #
!# Software Name : UVSOR ver. 3.00                                       #
!#                                                                       #
!#      Sub Routine : Transition_moment_Epsilon                          #
!#                                                                       #
!#                                Written by T. Hamada 2006/4/28         #
!#                                                                       #
!#      Contact address :  IIS,The University of Tokyo RSS21 project     #
!#                                                                       #
!#"Multiscale Simulation System for Functional Analysis of Nanomaterials"#
!#                                                                       #
!#========================================================================
!
!
! $Id: Transition_moment_Epsilon.f90 376 2014-06-17 07:48:31Z jkoga $
   subroutine Transition_moment_Epsilon
     use m_Epsilon_ek,            only : eigen_value_ordering_eps_ek &
        &                              ,calc_transition_moment_eps_ek
     call eigen_value_ordering_eps_ek
     call calc_transition_moment_eps_ek
   end subroutine Transition_moment_Epsilon
